local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.EnemyDetected, x.EnemyDetected, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

a.spells["auto_attack"] = {
	ID = 6603--[[Auto Attack]],
	EnemyTargetNeeded = 1,
	Melee = 1,
	NotIfActive = 1,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.EnemyDetected, "White", "Pink")
		return x.NoCC
	end,
}

a.spells["shadow_bolt,if=buff.shadowy_inspiration.remains&soul_shard<5&!prev_gcd.1.doom&!variable.no_de2"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 196606--[[Shadowy Inspiration]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.SoulShards < 5 and not x.LastCastSpell == 603--[[Doom]] and not x.no_de2 and not x.TalentDemonboltEnabled
	end,
}

a.spells["shadow_bolt,if=buff.shadowy_inspiration.remains"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 196606--[[Shadowy Inspiration]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return not s.Casting(686--[[Shadow Bolt]], "player") and not x.TalentDemonboltEnabled
	end,
}

a.spells["shadow_bolt"] = {
	ID = 686--[[Shadow Bolt]],
	NotWhileMoving = true,
	Check = function(z)
		return not x.TalentDemonboltEnabled
	end,
}

a.spells["demonic_empowerment,if=(((talent.power_trip.enabled&(!talent.implosion.enabled|spell_targets.demonwrath<=1))|!talent.implosion.enabled|(talent.implosion.enabled&!talent.soul_conduit.enabled&spell_targets.demonwrath<=3))&(wild_imp_no_de>3|prev_gcd.1.hand_of_guldan))|(prev_gcd.1.hand_of_guldan&wild_imp_no_de=0&wild_imp_remaining_duration<=0)|(prev_gcd.1.implosion&wild_imp_no_de>0)"] = {
	ID = 193396--[[Demonic Empowerment]],
	NoRangeCheck = true,
	NotWhileMoving = true,
	Check = function(z)
		return not s.Casting(193396--[[Demonic Empowerment]], "player") and ((((x.TalentPowerTripEnabled and (not x.TalentImplosionEnabled or x.ActiveEnemies <= 1)) or not x.TalentImplosionEnabled or (x.TalentImplosionEnabled and not x.TalentSoulConduitEnabled and x.ActiveEnemies <= 3)) and (a.wild_imp_no_de() > 3 or x.PrevGCDHandOfGuldan)) or (x.PrevGCDHandOfGuldan and a.wild_imp_no_de() == 0 --[[and wild_imp_remaining_duration<=0]]) or (x.LastCastSpell == 196277--[[Implosion]] and a.wild_imp_no_de() > 0))
	end,
}

a.spells["demonic_empowerment,if=variable.no_de1|prev_gcd.1.hand_of_guldan"] = {
	ID = 193396--[[Demonic Empowerment]],
	NoRangeCheck = true,
	NotWhileMoving = true,
	Check = function(z)
		return not s.Casting(193396--[[Demonic Empowerment]], "player") and (x.no_de1 or x.PrevGCDHandOfGuldan)
	end,
}

a.spells["demonic_empowerment,if=artifact.thalkiels_ascendance.rank&talent.power_trip.enabled&!talent.demonbolt.enabled&talent.shadowy_inspiration.enabled"] = {
	ID = 193396--[[Demonic Empowerment]],
	NoRangeCheck = true,
	NotWhileMoving = true,
	Check = function(z)
		return x.InCombat and not s.Casting(193396--[[Demonic Empowerment]], "player") and x.TalentPowerTripEnabled and not x.TalentDemonboltEnabled and x.TalentShadowyInspirationEnabled
	end,
}

a.spells["call_dreadstalkers,if=((!talent.summon_darkglare.enabled|talent.power_trip.enabled)&(spell_targets.implosion<3|!talent.implosion.enabled))&!(soul_shard=5&buff.demonic_calling.remains)"] = {
	ID = 104316--[[Call Dreeadstalkers]],
	NotWhileMoving = true,
	Check = function(z)
		return x.CanCastDreadstalkers and ((not x.TalentDarkglareEnabled or x.TalentPowerTripEnabled) and (x.ActiveEnemies < 3 or not x.TalentImplosionEnabled)) and not (x.SoulShards == 5 and x.BuffDemonicCallingRemains > x.GCD)
	end,
}

a.spells["call_dreadstalkers,if=talent.summon_darkglare.enabled&(spell_targets.implosion<3|!talent.implosion.enabled)&(cooldown.summon_darkglare.remains>2|prev_gcd.1.summon_darkglare|cooldown.summon_darkglare.remains<=action.call_dreadstalkers.cast_time&soul_shard>=3|cooldown.summon_darkglare.remains<=action.call_dreadstalkers.cast_time&soul_shard>=1&buff.demonic_calling.react)"] = {
	ID = 104316--[[Call Dreeadstalkers]],
	NotWhileMoving = true,
	Check = function(z)
		local call_dreadstalkers_cast_time = s.CastTime(104316--[[Call Dreadstalkers]])
		return x.CanCastDreadstalkers and x.TalentDarkglareEnabled and (x.ActiveEnemies < 3 or not x.TalentImplosionEnabled) and (x.CooldownSummonDarkglareRemains > 2 + x.GCD or x.LastCastSpell == 205180--[[Summon Darkglare]] or x.CooldownSummonDarkglareRemains <= call_dreadstalkers_cast_time and x.SoulShards >= 3 or x.CooldownSummonDarkglareRemains <= call_dreadstalkers_cast_time and x.SoulShards >= 1 and x.BuffDemonicCallingRemains > x.GCD)
	end,
}

a.spells["service_pet"] = {
	ID = 111898--[[Grimoire: Felguard]],
	NoRangeCheck = true,
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.InCombat
	end,
}

a.spells["doom,cycle_targets=1,if=(!talent.hand_of_doom.enabled&target.time_to_die>duration&(!ticking|remains<duration*0.3))&!(variable.no_de1|prev_gcd.1.hand_of_guldan)"] = {
	ID = 603--[[Doom]],
	Check = function(z)
		return (not s.HasTalent(196283--[[Hand of Doom]]) and x.TimeToDie > x.DoomDuration and x.DebuffDoomRemains < x.DoomDuration * 0.3) and not (x.no_de1 or x.PrevGCDHandOfGuldan)
	end,
}

a.spells["hand_of_guldan,if=soul_shard>=4&(((!(variable.no_de1|prev_gcd.1.hand_of_guldan)&(pet_count>=13&!talent.shadowy_inspiration.enabled|pet_count>=6&talent.shadowy_inspiration.enabled))|!variable.no_de2|soul_shard=5)&talent.power_trip.enabled)"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return not x.PrevGCDHandOfGuldan and x.SoulShards >= 4 and (((not (x.no_de1 or x.PrevGCDHandOfGuldan) and (a.getPetCount() >= 13 and not x.TalentShadowyInspirationEnabled or a.getPetCount() >= 6 and x.TalentShadowyInspirationEnabled)) or not x.no_de2 or x.SoulShards == 5) and x.TalentPowerTripEnabled)
	end,
}

a.spells["hand_of_guldan,if=(soul_shard>=3&prev_gcd.1.call_dreadstalkers&!artifact.thalkiels_ascendance.rank)|soul_shard>=5|(soul_shard>=4&cooldown.summon_darkglare.remains>2)"] = {
	ID = 105174--[[Hand of Gul'dan]],
	NotWhileMoving = true,
	Check = function(z)
		return not x.PrevGCDHandOfGuldan and ((x.SoulShards >= 3 and x.LastCastSpell == 104316--[[Summon Dreadstalkers]] and false) or x.SoulShards >= 5 or (x.SoulShards >= 4 and x.CooldownSummonDarkglareRemains > 2))
	end,
}

a.spells["soul_harvest,if=!buff.soul_harvest.remains"] = {
	ID = 196098--[[Soul Harvest]],
	Buff = 196098--[[Soul Harvest]],
	BuffUnit = "player",
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["felstorm"] = {
	ID = 119914--[[Felstorm]],
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
	Check = function(z)
		return x.InCombat and x.SoulShards >= 1
	end,
}

a.spells["berserking"] = {
	ID = 26297--[[Berserkingt]],
	Buff = 26297--[[Berserkingt]],
	BuffUnit = "player",
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
	Check = function(z)
		return s.Boss("target")
	end,
}

a.spells["life_tap,if=mana.pct<=15|(mana.pct<=65&((cooldown.call_dreadstalkers.remains<=0.75&soul_shard>=2)|((cooldown.call_dreadstalkers.remains<gcd*2)&(cooldown.summon_doomguard.remains<=0.75|cooldown.service_pet.remains<=0.75)&soul_shard>=3)))"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ManaPercent <= 15 or (x.ManaPercent <= 65 and ((x.CooldownCallDreadstalkers <= 0.75 and x.SoulShards >= 2) or ((x.CooldownCallDreadstalkers < x.MaxGCD * 2) and (s.SpellCooldown(18540--[[Summon Doomguard]]) <= 0.75 + x.GCD or s.SpellCooldown(111898--[[Grimoire: Felguard]]) <= 0.75 + x.GCD) and x.SoulShards >= 3)))
	end,
}

a.spells["life_tap"] = {
	ID = 1454--[[Life Tap]],
	Check = function(z)
		return x.InCombat
	end,
}

a.spells["thalkiels_consumption,if=(dreadstalker_remaining_duration>execute_time|talent.implosion.enabled&spell_targets.implosion>=3)&(wild_imp_count>3&dreadstalker_count<=2|wild_imp_count>5)&wild_imp_remaining_duration>execute_time"] = {
	ID = 211714--[[Thal'kiel's Consumption]],
	Check = function(z)
		return (a.dreadstalker_remaining_duration() > s.CastTime(211714--[[Thal'kiel's Consumption]]) or x.TalentImplosionEnabled and x.ActiveEnemies >= 3) and (a.wild_imp_count() > 3 and a.dreadstalker_count() <= 2 or a.wild_imp_count() > 5) and a.wild_imp_remaining_duration() > s.CastTime(211714--[[Thal'kiel's Consumption]])
	end,
}

a.spells["summon_doomguard,if=!talent.grimoire_of_supremacy.enabled&spell_targets.infernal_awakening<=2&(target.time_to_die>180|target.health.pct<=20|target.time_to_die<30)"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.SoulShards >= 1 and not x.TalentSupremacyEnabled and x.ActiveEnemies <= 2 and (x.TimeToDie > 180 or s.HealthPercent("target") <= 20 or x.TimeToDie < 30)
	end,
	FlashColor = "Pink",
	Continue = true,
}

a.spells["summon_infernal,if=!talent.grimoire_of_supremacy.enabled&spell_targets.infernal_awakening>2"] = {
	ID = 1122--[[Summon Infernal]],
	Check = function(z)
		return x.SoulShards >= 1 and not x.TalentSupremacyEnabled and x.ActiveEnemies > 2
	end,
	FlashColor = "Pink",
	Continue = true,
}

a.spells["demonwrath,chain=1,interrupt=1,if=spell_targets.demonwrath>=3"] = {
	ID = 193440--[[Demonwrath]],
	NoRangeCheck = true,
	Check = function(z)
		if s.Channeling(193440--[[Demonwrath]], "player") then
			z.FlashColor = "Purple"
		else
			z.FlashColor = "White"
		end
		return x.InCombat and x.ActiveEnemies >= 3
	end,
}

a.spells["demonwrath,moving=1,chain=1,interrupt=1"] = {
	ID = 193440--[[Demonwrath]],
	NoRangeCheck = true,
	Check = function(z)
		if s.Channeling(193440--[[Demonwrath]], "player") then
			z.FlashColor = "Purple"
		else
			z.FlashColor = "White"
		end
		return x.InCombat and s.Moving("player")
	end,
}

a.spells["demonbolt"] = {
	ID = 157695--[[Demonbolt]],
	NotWhileMoving = true,
	Check = function(z)
		return x.TalentDemonboltEnabled
	end,
}

a.spells["implosion,if=wild_imp_remaining_duration<=action.shadow_bolt.execute_time&(buff.demonic_synergy.remains|talent.soul_conduit.enabled|(!talent.soul_conduit.enabled&spell_targets.implosion>1)|wild_imp_count<=4)"] = {
	ID = 196277--[[Implosion]],
	Check = function(z)
		return x.TalentImplosionEnabled and a.wild_imp_count() > 0 and a.wild_imp_remaining_duration() <= s.CastTime(686--[[Shadow Bolt]]) + x.GCD and (x.BuffDemonicSynergyRemains > x.GCD or x.TalentSoulConduitEnabled or (not x.TalentSoulConduitEnabled and x.ActiveEnemies > 1) or a.wild_imp_count() <= 4)
	end,
}

a.spells["implosion,if=prev_gcd.1.hand_of_guldan&((wild_imp_remaining_duration<=3&buff.demonic_synergy.remains)|(wild_imp_remaining_duration<=4&spell_targets.implosion>2))"] = {
	ID = 196277--[[Implosion]],
	Check = function(z)
		return x.TalentImplosionEnabled and a.wild_imp_count() > 0 and x.PrevGCDHandOfGuldan and ((a.wild_imp_remaining_duration() <= 3 + x.GCD and x.BuffDemonicSynergyRemains > x.GCD) or (a.wild_imp_remaining_duration() <= 4 + x.GCD and x.ActiveEnemies > 2))
	end,
}
